diff --git a/src/core/src/main/java/org/apache/jmeter/testelement/AbstractTestElement.java b/src/core/src/main/java/org/apache/jmeter/testelement/AbstractTestElement.java
index 93f8369515..571c2a6ac8 100644
--- a/src/core/src/main/java/org/apache/jmeter/testelement/AbstractTestElement.java
+++ b/src/core/src/main/java/org/apache/jmeter/testelement/AbstractTestElement.java
@@ -26,6 +26,7 @@ import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.ConcurrentSkipListSet;
 
 import org.apache.jmeter.gui.Searchable;
 import org.apache.jmeter.testelement.property.BooleanProperty;
@@ -42,6 +43,7 @@ import org.apache.jmeter.testelement.property.StringProperty;
 import org.apache.jmeter.testelement.property.TestElementProperty;
 import org.apache.jmeter.threads.JMeterContext;
 import org.apache.jmeter.threads.JMeterContextService;
+import org.jetbrains.annotations.NotNull;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -67,6 +69,8 @@ public abstract class AbstractTestElement implements TestElement, Serializable,
 
     private transient String threadName = null;
 
+    private transient ConcurrentSkipListSet<ComparableThread> currentThreads = new ConcurrentSkipListSet<>();
+
     @Override
     public Object clone() {
         try {
@@ -502,6 +506,12 @@ public abstract class AbstractTestElement implements TestElement, Serializable,
      */
     @Override
     public void recoverRunningVersion() {
+        currentThreads.add(new ComparableThread(Thread.currentThread()));
+        if (currentThreads.size() > 1) {
+            log.warn("Recover in different threads: {}; Element: {}",
+                    currentThreads, this,
+                    new RuntimeException("Thread mismatch"));
+        }
         Iterator<Map.Entry<String, JMeterProperty>>  iter = propMap.entrySet().iterator();
         while (iter.hasNext()) {
             Map.Entry<String, JMeterProperty> entry = iter.next();
@@ -514,8 +524,29 @@ public abstract class AbstractTestElement implements TestElement, Serializable,
             }
         }
         emptyTemporary();
+        try {
+            Thread.sleep(500);
+        } catch (InterruptedException e) {
+            // nothing to do
+        }
     }
 
+    private static class ComparableThread extends Thread implements Comparable {
+        final Thread t;
+        ComparableThread(Thread t) {
+            this.t = t;
+        }
+
+        @Override
+        public int compareTo(@NotNull Object o) {
+            return this.t.getName().compareTo(((Thread) o).getName()) ;
+        }
+
+        @Override
+        public boolean equals(Object o) {
+            return o instanceof ComparableThread && compareTo((ComparableThread) o) == 0;
+        }
+    }
     /**
      * Clears temporaryProperties
      */
